import 'dart:async';

import 'package:flutter/services.dart';

class PagseguroSdk {
  static const MethodChannel _channel = const MethodChannel('pagseguro_sdk');
  static const EventChannel _eventsChannel = const EventChannel('pagseguro_events');
  static Stream<dynamic> _eventsStream;

  Future<String> get platformVersion async {
    print('PAGSEGURO: ' + _channel.name);
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  Future<bool> initialize({String appVersion, String appName, String activationCode}) async {

    final dynamic result = await _channel.invokeMethod('initialize', {
      'appName': appName,
      'appVersion': appVersion,
      'activationCode': activationCode
    });

    return result;
  }

  void subscribeToEvents(Null Function(dynamic event) callback) {
    _eventsStream = _eventsChannel.receiveBroadcastStream();
    _eventsStream.listen((event) {
      print('..................................... event! $event');
      callback(event);
    });
  }

  Future<List<dynamic>> getInstallments({num amount}) {
    return _channel.invokeListMethod('getInstallments', {
      'amount': amount
    });
  }

  Future<bool> startPayment({
    num amount,
    int installments,
    String type, Null Function(dynamic event) listener
  }) async {
    await _channel.invokeMethod('startPayment', {
      'amount': amount,
      'installments': installments,
      'type': type,
      'paymentId': '',
      'listener': listener
    });
    return true;
  }
}
